@@ -37,6 +37,7 @@
 #include "miscstruct.h"
 #include "servermd.h"
 #include "windowstr.h"
+#include "privates.h"
 #include "mi.h"
 #include "migc.h"
 #include "mibstore.h"
@@ -100,9 +101,6 @@
 #if GLYPHPADBYTES != 4
 #error "GLYPHPADBYTES must be 4"
 #endif
-#if GETLEFTBITS_ALIGNMENT != 1
-#error "GETLEFTBITS_ALIGNMENT must be 1"
-#endif
 /* whether to bother to include 24bpp support */
 #ifndef FBNO24BIT
 #define FB_24BIT
@@ -141,8 +139,7 @@ typedef unsigned __int64    FbBits;
       defined(__sparc64__) || defined(_LP64) || \
       defined(__s390x__) || \
       defined(amd64) || defined (__amd64__) || \
-      defined (__powerpc64__) || \
-      (defined(sgi) && (_MIPS_SZLONG == 64))
+      defined (__powerpc64__)
 typedef unsigned long	    FbBits;
 #  else
 typedef unsigned long long  FbBits;
@@ -170,9 +167,9 @@ typedef int		    FbStride;
 
 
 #ifdef FB_DEBUG
-extern void fbValidateDrawable(DrawablePtr d);
-extern void fbInitializeDrawable(DrawablePtr d);
-extern void fbSetBits (FbStip *bits, int stride, FbStip data);
+extern _X_EXPORT void fbValidateDrawable(DrawablePtr d);
+extern _X_EXPORT void fbInitializeDrawable(DrawablePtr d);
+extern _X_EXPORT void fbSetBits (FbStip *bits, int stride, FbStip data);
 #define FB_HEAD_BITS   (FbStip) (0xbaadf00d)
 #define FB_TAIL_BITS   (FbStip) (0xbaddf0ad)
 #else
@@ -490,22 +487,34 @@ extern void fbSetBits (FbStip *bits, int stride, FbStip data);
  * The term "lane" comes from the hardware term "byte-lane" which
  */
 
-#define FbLaneCase1(n,a,o)  ((n) == 0x01 ? (void) \
-			     WRITE((CARD8 *) ((a)+FbPatternOffset(o,CARD8)), \
-			      fgxor) : (void) 0)
-#define FbLaneCase2(n,a,o)  ((n) == 0x03 ? (void) \
-			     WRITE((CARD16 *) ((a)+FbPatternOffset(o,CARD16)), \
-			      fgxor) : \
-			     ((void)FbLaneCase1((n)&1,a,o), \
-				    FbLaneCase1((n)>>1,a,(o)+1)))
-#define FbLaneCase4(n,a,o)  ((n) == 0x0f ? (void) \
-			     WRITE((CARD32 *) ((a)+FbPatternOffset(o,CARD32)), \
-			      fgxor) : \
-			     ((void)FbLaneCase2((n)&3,a,o), \
-				    FbLaneCase2((n)>>2,a,(o)+2)))
-#define FbLaneCase8(n,a,o)  ((n) == 0x0ff ? (void) (*(FbBits *) ((a)+(o)) = fgxor) : \
-			     ((void)FbLaneCase4((n)&15,a,o), \
-				    FbLaneCase4((n)>>4,a,(o)+4)))
+#define FbLaneCase1(n,a,o)						\
+    if ((n) == 0x01) {							\
+	WRITE((CARD8 *) ((a)+FbPatternOffset(o,CARD8)), fgxor);		\
+    }
+
+#define FbLaneCase2(n,a,o)						\
+    if ((n) == 0x03) {							\
+	WRITE((CARD16 *) ((a)+FbPatternOffset(o,CARD16)), fgxor);	\
+    } else {								\
+	FbLaneCase1((n)&1,a,o)						\
+	FbLaneCase1((n)>>1,a,(o)+1)					\
+    }
+
+#define FbLaneCase4(n,a,o)						\
+    if ((n) == 0x0f) {							\
+	WRITE((CARD32 *) ((a)+FbPatternOffset(o,CARD32)), fgxor);	\
+    } else {								\
+	FbLaneCase2((n)&3,a,o)						\
+	FbLaneCase2((n)>>2,a,(o)+2)					\
+    }
+
+#define FbLaneCase8(n,a,o)						\
+    if ((n) == 0x0ff) {							\
+	*(FbBits *) ((a)+(o)) = fgxor;					\
+    } else {								\
+	FbLaneCase4((n)&15,a,o)						\
+	FbLaneCase4((n)>>4,a,(o)+4)					\
+    }
 
 #if FB_SHIFT == 6
 #define FbLaneCase(n,a)   FbLaneCase8(n,(CARD8 *) (a),0)
@@ -598,16 +607,10 @@ extern void fbSetBits (FbStip *bits, int stride, FbStip data);
     }							    \
 }
 
-/* XXX fb*PrivateIndex should be static, but it breaks the ABI */
-
-extern int	fbGCPrivateIndex;
-extern int	fbGetGCPrivateIndex(void);
-#ifndef FB_NO_WINDOW_PIXMAPS
-extern int	fbWinPrivateIndex;
-extern int	fbGetWinPrivateIndex(void);
-#endif
-extern const GCOps	fbGCOps;
-extern const GCFuncs	fbGCFuncs;
+extern _X_EXPORT DevPrivateKey fbGetGCPrivateKey(void);
+extern _X_EXPORT DevPrivateKey fbGetWinPrivateKey(void);
+extern _X_EXPORT const GCOps	fbGCOps;
+extern _X_EXPORT const GCFuncs	fbGCFuncs;
 
 #ifdef FB_24_32BIT
 #define FB_SCREEN_PRIVATE
@@ -640,8 +643,7 @@ typedef void (*FinishWrapProcPtr)(DrawablePtr pDraw);
 
 
 #ifdef FB_SCREEN_PRIVATE
-extern int	fbScreenPrivateIndex;
-extern int	fbGetScreenPrivateIndex(void);
+extern _X_EXPORT DevPrivateKey fbGetScreenPrivateKey(void);
 
 /* private field of a screen */
 typedef struct {
@@ -654,7 +656,7 @@ typedef struct {
 } FbScreenPrivRec, *FbScreenPrivPtr;
 
 #define fbGetScreenPrivate(pScreen) ((FbScreenPrivPtr) \
-				     (pScreen)->devPrivates[fbGetScreenPrivateIndex()].ptr)
+	dixLookupPrivate(&(pScreen)->devPrivates, fbGetScreenPrivateKey()))
 #endif
 
 /* private field of GC */
@@ -669,7 +671,7 @@ typedef struct {
 } FbGCPrivRec, *FbGCPrivPtr;
 
 #define fbGetGCPrivate(pGC)	((FbGCPrivPtr)\
-	(pGC)->devPrivates[fbGetGCPrivateIndex()].ptr)
+	dixLookupPrivate(&(pGC)->devPrivates, fbGetGCPrivateKey()))
 
 #define fbGetCompositeClip(pGC) ((pGC)->pCompositeClip)
 #define fbGetExpose(pGC)	((pGC)->fExpose)
@@ -677,12 +679,8 @@ typedef struct {
 #define fbGetRotatedPixmap(pGC)	((pGC)->pRotatedPixmap)
 
 #define fbGetScreenPixmap(s)	((PixmapPtr) (s)->devPrivate)
-#ifdef FB_NO_WINDOW_PIXMAPS
-#define fbGetWindowPixmap(d)	fbGetScreenPixmap(((DrawablePtr) (d))->pScreen)
-#else
 #define fbGetWindowPixmap(pWin)	((PixmapPtr)\
-	((WindowPtr) (pWin))->devPrivates[fbGetWinPrivateIndex()].ptr)
-#endif
+    dixLookupPrivate(&((WindowPtr)(pWin))->devPrivates, fbGetWinPrivateKey()))
 
 #ifdef ROOTLESS
 #define __fbPixDrawableX(pPix)	((pPix)->drawable.x)
@@ -763,7 +761,7 @@ typedef struct {
 /*
  * fb24_32.c
  */
-void
+extern _X_EXPORT void
 fb24_32GetSpans(DrawablePtr	pDrawable, 
 		int		wMax, 
 		DDXPointPtr	ppt, 
@@ -771,7 +769,7 @@ fb24_32GetSpans(DrawablePtr	pDrawable,
 		int		nspans, 
 		char		*pchardstStart);
 
-void
+extern _X_EXPORT void
 fb24_32SetSpans (DrawablePtr	    pDrawable,
 		 GCPtr		    pGC,
 		 char		    *src,
@@ -780,7 +778,7 @@ fb24_32SetSpans (DrawablePtr	    pDrawable,
 		 int		    nspans,
 		 int		    fSorted);
 
-void
+extern _X_EXPORT void
 fb24_32PutZImage (DrawablePtr	pDrawable,
 		  RegionPtr	pClip,
 		  int		alu,
@@ -792,7 +790,7 @@ fb24_32PutZImage (DrawablePtr	pDrawable,
 		  CARD8		*src,
 		  FbStride	srcStride);
     
-void
+extern _X_EXPORT void
 fb24_32GetImage (DrawablePtr     pDrawable,
 		 int             x,
 		 int             y,
@@ -802,7 +800,7 @@ fb24_32GetImage (DrawablePtr     pDrawable,
 		 unsigned long   planeMask,
 		 char            *d);
 
-void
+extern _X_EXPORT void
 fb24_32CopyMtoN (DrawablePtr pSrcDrawable,
 		 DrawablePtr pDstDrawable,
 		 GCPtr       pGC,
@@ -815,13 +813,13 @@ fb24_32CopyMtoN (DrawablePtr pSrcDrawable,
 		 Pixel       bitplane,
 		 void        *closure);
 
-PixmapPtr
+extern _X_EXPORT PixmapPtr
 fb24_32ReformatTile(PixmapPtr pOldTile, int bitsPerPixel);
     
-Bool
+extern _X_EXPORT Bool
 fb24_32CreateScreenResources(ScreenPtr pScreen);
 
-Bool
+extern _X_EXPORT Bool
 fb24_32ModifyPixmapHeader (PixmapPtr   pPixmap,
 			   int         width,
 			   int         height,
@@ -833,14 +831,16 @@ fb24_32ModifyPixmapHeader (PixmapPtr   pPixmap,
 /*
  * fballpriv.c
  */
-Bool
-fbAllocatePrivates(ScreenPtr pScreen, int *pGCIndex);
+extern _X_EXPORT DevPrivateKey fbGetWinPrivateKey(void);
+
+extern _X_EXPORT Bool
+fbAllocatePrivates(ScreenPtr pScreen, DevPrivateKey *pGCIndex);
     
 /*
  * fbarc.c
  */
 
-void
+extern _X_EXPORT void
 fbPolyArc (DrawablePtr	pDrawable,
 	   GCPtr	pGC,
 	   int		narcs,
@@ -850,7 +850,7 @@ fbPolyArc (DrawablePtr	pDrawable,
  * fbbits.c
  */
 
-void	
+extern _X_EXPORT void
 fbBresSolid8(DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    dashOffset,
@@ -864,7 +864,7 @@ fbBresSolid8(DrawablePtr    pDrawable,
 	     int	    e3,
 	     int	    len);
 
-void	
+extern _X_EXPORT void
 fbBresDash8 (DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    dashOffset,
@@ -878,7 +878,7 @@ fbBresDash8 (DrawablePtr    pDrawable,
 	     int	    e3,
 	     int	    len);
 
-void	
+extern _X_EXPORT void
 fbDots8 (FbBits	    *dst,
 	 FbStride   dstStride,
 	 int	    dstBpp,
@@ -892,7 +892,7 @@ fbDots8 (FbBits	    *dst,
 	 FbBits	    and,
 	 FbBits	    xor);
 
-void	
+extern _X_EXPORT void
 fbArc8 (FbBits	    *dst,
 	FbStride    dstStride,
 	int	    dstBpp,
@@ -902,7 +902,7 @@ fbArc8 (FbBits	    *dst,
 	FbBits	    and,
 	FbBits	    xor);
 
-void
+extern _X_EXPORT void
 fbGlyph8 (FbBits    *dstLine,
 	  FbStride  dstStride,
 	  int	    dstBpp,
@@ -911,20 +911,20 @@ fbGlyph8 (FbBits    *dstLine,
 	  int	    height,
 	  int	    shift);
 
-void
+extern _X_EXPORT void
 fbPolyline8 (DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    mode,
 	     int	    npt,
 	     DDXPointPtr    ptsOrig);
 
-void
+extern _X_EXPORT void
 fbPolySegment8 (DrawablePtr pDrawable,
 		GCPtr	    pGC,
 		int	    nseg,
 		xSegment    *pseg);
 
-void	
+extern _X_EXPORT void
 fbBresSolid16(DrawablePtr   pDrawable,
 	      GCPtr	    pGC,
 	      int	    dashOffset,
@@ -938,7 +938,7 @@ fbBresSolid16(DrawablePtr   pDrawable,
 	      int	    e3,
 	      int	    len);
 
-void	
+extern _X_EXPORT void
 fbBresDash16(DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    dashOffset,
@@ -952,7 +952,7 @@ fbBresDash16(DrawablePtr    pDrawable,
 	     int	    e3,
 	     int	    len);
 
-void	
+extern _X_EXPORT void
 fbDots16(FbBits	    *dst,
 	 FbStride   dstStride,
 	 int	    dstBpp,
@@ -966,7 +966,7 @@ fbDots16(FbBits	    *dst,
 	 FbBits	    and,
 	 FbBits	    xor);
 
-void	
+extern _X_EXPORT void
 fbArc16(FbBits	    *dst,
 	FbStride    dstStride,
 	int	    dstBpp,
@@ -976,7 +976,7 @@ fbArc16(FbBits	    *dst,
 	FbBits	    and,
 	FbBits	    xor);
 
-void
+extern _X_EXPORT void
 fbGlyph16(FbBits    *dstLine,
 	  FbStride  dstStride,
 	  int	    dstBpp,
@@ -985,21 +985,21 @@ fbGlyph16(FbBits    *dstLine,
 	  int	    height,
 	  int	    shift);
 
-void
+extern _X_EXPORT void
 fbPolyline16 (DrawablePtr   pDrawable,
 	      GCPtr	    pGC,
 	      int	    mode,
 	      int	    npt,
 	      DDXPointPtr   ptsOrig);
 
-void
+extern _X_EXPORT void
 fbPolySegment16 (DrawablePtr	pDrawable,
 		 GCPtr		pGC,
 		 int		nseg,
 		 xSegment	*pseg);
 
 
-void	
+extern _X_EXPORT void
 fbBresSolid24(DrawablePtr   pDrawable,
 	      GCPtr	    pGC,
 	      int	    dashOffset,
@@ -1013,7 +1013,7 @@ fbBresSolid24(DrawablePtr   pDrawable,
 	      int	    e3,
 	      int	    len);
 
-void	
+extern _X_EXPORT void
 fbBresDash24(DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    dashOffset,
@@ -1027,7 +1027,7 @@ fbBresDash24(DrawablePtr    pDrawable,
 	     int	    e3,
 	     int	    len);
 
-void	
+extern _X_EXPORT void
 fbDots24(FbBits	    *dst,
 	 FbStride   dstStride,
 	 int	    dstBpp,
@@ -1041,7 +1041,7 @@ fbDots24(FbBits	    *dst,
 	 FbBits	    and,
 	 FbBits	    xor);
 
-void	
+extern _X_EXPORT void
 fbArc24(FbBits	    *dst,
 	FbStride    dstStride,
 	int	    dstBpp,
@@ -1051,7 +1051,7 @@ fbArc24(FbBits	    *dst,
 	FbBits	    and,
 	FbBits	    xor);
 
-void
+extern _X_EXPORT void
 fbGlyph24(FbBits    *dstLine,
 	  FbStride  dstStride,
 	  int	    dstBpp,
@@ -1060,21 +1060,21 @@ fbGlyph24(FbBits    *dstLine,
 	  int	    height,
 	  int	    shift);
 
-void
+extern _X_EXPORT void
 fbPolyline24 (DrawablePtr   pDrawable,
 	      GCPtr	    pGC,
 	      int	    mode,
 	      int	    npt,
 	      DDXPointPtr   ptsOrig);
 
-void
+extern _X_EXPORT void
 fbPolySegment24 (DrawablePtr	pDrawable,
 		 GCPtr		pGC,
 		 int		nseg,
 		 xSegment	*pseg);
 
 
-void	
+extern _X_EXPORT void
 fbBresSolid32(DrawablePtr   pDrawable,
 	      GCPtr	    pGC,
 	      int	    dashOffset,
@@ -1088,7 +1088,7 @@ fbBresSolid32(DrawablePtr   pDrawable,
 	      int	    e3,
 	      int	    len);
 
-void	
+extern _X_EXPORT void
 fbBresDash32(DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    dashOffset,
@@ -1102,7 +1102,7 @@ fbBresDash32(DrawablePtr    pDrawable,
 	     int	    e3,
 	     int	    len);
 
-void	
+extern _X_EXPORT void
 fbDots32(FbBits	    *dst,
 	 FbStride   dstStride,
 	 int	    dstBpp,
@@ -1116,7 +1116,7 @@ fbDots32(FbBits	    *dst,
 	 FbBits	    and,
 	 FbBits	    xor);
 
-void	
+extern _X_EXPORT void
 fbArc32(FbBits	    *dst,
 	FbStride    dstStride,
 	int	    dstBpp,
@@ -1126,7 +1126,7 @@ fbArc32(FbBits	    *dst,
 	FbBits	    and,
 	FbBits	    xor);
 
-void
+extern _X_EXPORT void
 fbGlyph32(FbBits    *dstLine,
 	  FbStride  dstStride,
 	  int	    dstBpp,
@@ -1134,14 +1134,14 @@ fbGlyph32(FbBits    *dstLine,
 	  FbBits    fg,
 	  int	    height,
 	  int	    shift);
-void
+extern _X_EXPORT void
 fbPolyline32 (DrawablePtr   pDrawable,
 	      GCPtr	    pGC,
 	      int	    mode,
 	      int	    npt,
 	      DDXPointPtr   ptsOrig);
 
-void
+extern _X_EXPORT void
 fbPolySegment32 (DrawablePtr	pDrawable,
 		 GCPtr		pGC,
 		 int		nseg,
@@ -1150,7 +1150,7 @@ fbPolySegment32 (DrawablePtr	pDrawable,
 /*
  * fbblt.c
  */
-void
+extern _X_EXPORT void
 fbBlt (FbBits   *src, 
        FbStride	srcStride,
        int	srcX,
@@ -1169,7 +1169,7 @@ fbBlt (FbBits   *src,
        Bool	reverse,
        Bool	upsidedown);
 
-void
+extern _X_EXPORT void
 fbBlt24 (FbBits	    *srcLine,
 	 FbStride   srcStride,
 	 int	    srcX,
@@ -1187,7 +1187,7 @@ fbBlt24 (FbBits	    *srcLine,
 	 Bool	    reverse,
 	 Bool	    upsidedown);
     
-void
+extern _X_EXPORT void
 fbBltStip (FbStip   *src,
 	   FbStride srcStride,	    /* in FbStip units, not FbBits units */
 	   int	    srcX,
@@ -1206,7 +1206,7 @@ fbBltStip (FbStip   *src,
 /*
  * fbbltone.c
  */
-void
+extern _X_EXPORT void
 fbBltOne (FbStip   *src,
 	  FbStride srcStride,
 	  int	   srcX,
@@ -1224,7 +1224,7 @@ fbBltOne (FbStip   *src,
 	  FbBits   bgxor);
  
 #ifdef FB_24BIT
-void
+extern _X_EXPORT void
 fbBltOne24 (FbStip    *src,
 	  FbStride  srcStride,	    /* FbStip units per scanline */
 	  int	    srcX,	    /* bit position of source */
@@ -1242,7 +1242,7 @@ fbBltOne24 (FbStip    *src,
 	  FbBits    bgxor);
 #endif
 
-void
+extern _X_EXPORT void
 fbBltPlane (FbBits	    *src,
 	    FbStride	    srcStride,
 	    int		    srcX,
@@ -1261,67 +1261,50 @@ fbBltPlane (FbBits	    *src,
 	    FbStip	    bgxor,
 	    Pixel	    planeMask);
 
-/*
- * fbbstore.c
- */
-void
-fbSaveAreas(PixmapPtr	pPixmap,
-	    RegionPtr	prgnSave,
-	    int		xorg,
-	    int		yorg,
-	    WindowPtr	pWin);
-
-void
-fbRestoreAreas(PixmapPtr    pPixmap,
-	       RegionPtr    prgnRestore,
-	       int	    xorg,
-	       int	    yorg,
-	       WindowPtr    pWin);
-
 /*
  * fbcmap.c
  */
-int
+extern _X_EXPORT int
 fbListInstalledColormaps(ScreenPtr pScreen, Colormap *pmaps);
 
-void
+extern _X_EXPORT void
 fbInstallColormap(ColormapPtr pmap);
 
-void
+extern _X_EXPORT void
 fbUninstallColormap(ColormapPtr pmap);
 
-void
+extern _X_EXPORT void
 fbResolveColor(unsigned short	*pred, 
 	       unsigned short	*pgreen, 
 	       unsigned short	*pblue,
 	       VisualPtr	pVisual);
 
-Bool
+extern _X_EXPORT Bool
 fbInitializeColormap(ColormapPtr pmap);
 
-int
+extern _X_EXPORT int
 fbExpandDirectColors (ColormapPtr   pmap, 
 		      int	    ndef,
 		      xColorItem    *indefs,
 		      xColorItem    *outdefs);
 
-Bool
+extern _X_EXPORT Bool
 fbCreateDefColormap(ScreenPtr pScreen);
 
-void
+extern _X_EXPORT void
 fbClearVisualTypes(void);
 
-Bool
+extern _X_EXPORT Bool
 fbHasVisualTypes (int depth);
 
-Bool
+extern _X_EXPORT Bool
 fbSetVisualTypes (int depth, int visuals, int bitsPerRGB);
 
-Bool
+extern _X_EXPORT Bool
 fbSetVisualTypesAndMasks (int depth, int visuals, int bitsPerRGB,
 			  Pixel redMask, Pixel greenMask, Pixel blueMask);
 
-Bool
+extern _X_EXPORT Bool
 fbInitVisuals (VisualPtr    *visualp, 
 	       DepthPtr	    *depthp,
 	       int	    *nvisualp,
@@ -1335,19 +1318,20 @@ fbInitVisuals (VisualPtr    *visualp,
  * fbcopy.c
  */
 
-typedef void	(*fbCopyProc) (DrawablePtr  pSrcDrawable,
-			       DrawablePtr  pDstDrawable,
-			       GCPtr	    pGC,
-			       BoxPtr	    pDstBox,
-			       int	    nbox,
-			       int	    dx,
-			       int	    dy,
-			       Bool	    reverse,
-			       Bool	    upsidedown,
-			       Pixel	    bitplane,
-			       void	    *closure);
+/* Compatibility definition, to be removed at next ABI change. */
+typedef void   (*fbCopyProc) (DrawablePtr  pSrcDrawable,
+                              DrawablePtr  pDstDrawable,
+                              GCPtr        pGC,
+                              BoxPtr       pDstBox,
+                              int          nbox,
+                              int          dx,
+                              int          dy,
+                              Bool         reverse,
+                              Bool         upsidedown,
+                              Pixel        bitplane,
+                              void         *closure);
 
-void
+extern _X_EXPORT void
 fbCopyNtoN (DrawablePtr	pSrcDrawable,
 	    DrawablePtr	pDstDrawable,
 	    GCPtr	pGC,
@@ -1360,7 +1344,34 @@ fbCopyNtoN (DrawablePtr	pSrcDrawable,
 	    Pixel	bitplane,
 	    void	*closure);
 
-void
+/* Compatibility wrapper, to be removed at next ABI change. */
+extern _X_EXPORT void
+fbCopyRegion (DrawablePtr   pSrcDrawable,
+             DrawablePtr   pDstDrawable,
+             GCPtr         pGC,
+             RegionPtr     pDstRegion,
+             int           dx,
+             int           dy,
+             fbCopyProc    copyProc,
+             Pixel         bitPlane,
+             void          *closure);
+
+/* Compatibility wrapper, to be removed at next ABI change. */
+extern _X_EXPORT RegionPtr
+fbDoCopy (DrawablePtr  pSrcDrawable,
+         DrawablePtr   pDstDrawable,
+         GCPtr         pGC,
+         int           xIn,
+         int           yIn,
+         int           widthSrc,
+         int           heightSrc,
+         int           xOut,
+         int           yOut,
+         fbCopyProc    copyProc,
+         Pixel         bitplane,
+         void          *closure);
+
+extern _X_EXPORT void
 fbCopy1toN (DrawablePtr	pSrcDrawable,
 	    DrawablePtr	pDstDrawable,
 	    GCPtr	pGC,
@@ -1373,7 +1384,7 @@ fbCopy1toN (DrawablePtr	pSrcDrawable,
 	    Pixel	bitplane,
 	    void	*closure);
 
-void
+extern _X_EXPORT void
 fbCopyNto1 (DrawablePtr	pSrcDrawable,
 	    DrawablePtr	pDstDrawable,
 	    GCPtr	pGC,
@@ -1386,32 +1397,7 @@ fbCopyNto1 (DrawablePtr	pSrcDrawable,
 	    Pixel	bitplane,
 	    void	*closure);
 
-void
-fbCopyRegion (DrawablePtr   pSrcDrawable,
-	      DrawablePtr   pDstDrawable,
-	      GCPtr	    pGC,
-	      RegionPtr	    pDstRegion,
-	      int	    dx,
-	      int	    dy,
-	      fbCopyProc    copyProc,
-	      Pixel	    bitPlane,
-	      void	    *closure);
-
-RegionPtr
-fbDoCopy (DrawablePtr	pSrcDrawable,
-	  DrawablePtr	pDstDrawable,
-	  GCPtr		pGC,
-	  int		xIn, 
-	  int		yIn,
-	  int		widthSrc, 
-	  int		heightSrc,
-	  int		xOut, 
-	  int		yOut,
-	  fbCopyProc	copyProc,
-	  Pixel		bitplane,
-	  void		*closure);
-	  
-RegionPtr
+extern _X_EXPORT RegionPtr
 fbCopyArea (DrawablePtr	pSrcDrawable,
 	    DrawablePtr	pDstDrawable,
 	    GCPtr	pGC,
@@ -1422,7 +1408,7 @@ fbCopyArea (DrawablePtr	pSrcDrawable,
 	    int		xOut, 
 	    int		yOut);
 
-RegionPtr
+extern _X_EXPORT RegionPtr
 fbCopyPlane (DrawablePtr    pSrcDrawable,
 	     DrawablePtr    pDstDrawable,
 	     GCPtr	    pGC,
@@ -1437,7 +1423,7 @@ fbCopyPlane (DrawablePtr    pSrcDrawable,
 /*
  * fbfill.c
  */
-void
+extern _X_EXPORT void
 fbFill (DrawablePtr pDrawable,
 	GCPtr	    pGC,
 	int	    x,
@@ -1445,7 +1431,7 @@ fbFill (DrawablePtr pDrawable,
 	int	    width,
 	int	    height);
 
-void
+extern _X_EXPORT void
 fbSolidBoxClipped (DrawablePtr	pDrawable,
 		   RegionPtr	pClip,
 		   int		xa,
@@ -1458,7 +1444,7 @@ fbSolidBoxClipped (DrawablePtr	pDrawable,
 /*
  * fbfillrect.c
  */
-void
+extern _X_EXPORT void
 fbPolyFillRect(DrawablePtr  pDrawable, 
 	       GCPtr	    pGC, 
 	       int	    nrectInit,
@@ -1471,7 +1457,7 @@ fbPolyFillRect(DrawablePtr  pDrawable,
 /*
  * fbfillsp.c
  */
-void
+extern _X_EXPORT void
 fbFillSpans (DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    nInit,
@@ -1484,19 +1470,19 @@ fbFillSpans (DrawablePtr    pDrawable,
  * fbgc.c
  */
 
-Bool
+extern _X_EXPORT Bool
 fbCreateGC(GCPtr pGC);
 
-void
+extern _X_EXPORT void
 fbPadPixmap (PixmapPtr pPixmap);
     
-void
+extern _X_EXPORT void
 fbValidateGC(GCPtr pGC, unsigned long changes, DrawablePtr pDrawable);
 
 /*
  * fbgetsp.c
  */
-void
+extern _X_EXPORT void
 fbGetSpans(DrawablePtr	pDrawable, 
 	   int		wMax, 
 	   DDXPointPtr	ppt, 
@@ -1508,14 +1494,14 @@ fbGetSpans(DrawablePtr	pDrawable,
  * fbglyph.c
  */
 
-Bool
+extern _X_EXPORT Bool
 fbGlyphIn (RegionPtr	pRegion,
 	   int		x,
 	   int		y,
 	   int		width,
 	   int		height);
     
-void
+extern _X_EXPORT void
 fbPolyGlyphBlt (DrawablePtr	pDrawable,
 		GCPtr		pGC,
 		int		x, 
@@ -1524,7 +1510,7 @@ fbPolyGlyphBlt (DrawablePtr	pDrawable,
 		CharInfoPtr	*ppci,
 		pointer		pglyphBase);
 
-void
+extern _X_EXPORT void
 fbImageGlyphBlt (DrawablePtr	pDrawable,
 		 GCPtr		pGC,
 		 int		x,
@@ -1537,7 +1523,7 @@ fbImageGlyphBlt (DrawablePtr	pDrawable,
  * fbimage.c
  */
 
-void
+extern _X_EXPORT void
 fbPutImage (DrawablePtr	pDrawable,
 	    GCPtr	pGC,
 	    int		depth,
@@ -1549,7 +1535,7 @@ fbPutImage (DrawablePtr	pDrawable,
 	    int		format,
 	    char	*pImage);
 
-void
+extern _X_EXPORT void
 fbPutZImage (DrawablePtr	pDrawable,
 	     RegionPtr		pClip,
 	     int		alu,
@@ -1561,7 +1547,7 @@ fbPutZImage (DrawablePtr	pDrawable,
 	     FbStip		*src,
 	     FbStride		srcStride);
 
-void
+extern _X_EXPORT void
 fbPutXYImage (DrawablePtr	pDrawable,
 	      RegionPtr		pClip,
 	      FbBits		fg,
@@ -1579,7 +1565,7 @@ fbPutXYImage (DrawablePtr	pDrawable,
 	      FbStride		srcStride,
 	      int		srcX);
 
-void
+extern _X_EXPORT void
 fbGetImage (DrawablePtr	    pDrawable,
 	    int		    x,
 	    int		    y,
@@ -1592,31 +1578,31 @@ fbGetImage (DrawablePtr	    pDrawable,
  * fbline.c
  */
 
-void
+extern _X_EXPORT void
 fbZeroLine (DrawablePtr	pDrawable,
 	    GCPtr	pGC,
 	    int		mode,
 	    int		npt,
 	    DDXPointPtr	ppt);
 
-void
+extern _X_EXPORT void
 fbZeroSegment (DrawablePtr  pDrawable,
 	       GCPtr	    pGC,
 	       int	    nseg,
 	       xSegment	    *pSegs);
 
-void
+extern _X_EXPORT void
 fbPolyLine (DrawablePtr	pDrawable,
 	    GCPtr	pGC,
 	    int		mode,
 	    int		npt,
 	    DDXPointPtr	ppt);
 
-void
+extern _X_EXPORT void
 fbFixCoordModePrevious (int npt,
 			DDXPointPtr ppt);
 
-void
+extern _X_EXPORT void
 fbPolySegment (DrawablePtr  pDrawable,
 	       GCPtr	    pGC,
 	       int	    nseg,
@@ -1628,7 +1614,7 @@ fbPolySegment (DrawablePtr  pDrawable,
  * fbpict.c
  */
 
-Bool
+extern _X_EXPORT Bool
 fbPictureInit (ScreenPtr pScreen,
 	       PictFormatPtr formats,
 	       int nformats);
@@ -1637,23 +1623,25 @@ fbPictureInit (ScreenPtr pScreen,
  * fbpixmap.c
  */
 
-PixmapPtr
-fbCreatePixmapBpp (ScreenPtr pScreen, int width, int height, int depth, int bpp);
+extern _X_EXPORT PixmapPtr
+fbCreatePixmapBpp (ScreenPtr pScreen, int width, int height, int depth, int bpp,
+		   unsigned usage_hint);
 
-PixmapPtr
-fbCreatePixmap (ScreenPtr pScreen, int width, int height, int depth);
+extern _X_EXPORT PixmapPtr
+fbCreatePixmap (ScreenPtr pScreen, int width, int height, int depth,
+		unsigned usage_hint);
 
-Bool
+extern _X_EXPORT Bool
 fbDestroyPixmap (PixmapPtr pPixmap);
 
-RegionPtr
+extern _X_EXPORT RegionPtr
 fbPixmapToRegion(PixmapPtr pPix);
 
 /*
  * fbpoint.c
  */
 
-void
+extern _X_EXPORT void
 fbDots (FbBits	    *dstOrig,
 	FbStride    dstStride,
 	int	    dstBpp,
@@ -1667,7 +1655,7 @@ fbDots (FbBits	    *dstOrig,
 	FbBits	    andOrig,
 	FbBits	    xorOrig);
 
-void
+extern _X_EXPORT void
 fbPolyPoint (DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     int	    mode,
@@ -1677,7 +1665,7 @@ fbPolyPoint (DrawablePtr    pDrawable,
 /*
  * fbpush.c
  */
-void
+extern _X_EXPORT void
 fbPushPattern (DrawablePtr  pDrawable,
 	       GCPtr	    pGC,
 	       
@@ -1691,7 +1679,7 @@ fbPushPattern (DrawablePtr  pDrawable,
 	       int	    width,
 	       int	    height);
 
-void
+extern _X_EXPORT void
 fbPushFill (DrawablePtr	pDrawable,
 	    GCPtr	pGC,
 
@@ -1704,20 +1692,7 @@ fbPushFill (DrawablePtr	pDrawable,
 	    int		width,
 	    int		height);
 
-void
-fbPush1toN (DrawablePtr	pSrcDrawable,
-	    DrawablePtr	pDstDrawable,
-	    GCPtr	pGC,
-	    BoxPtr	pbox,
-	    int		nbox,
-	    int		dx,
-	    int		dy,
-	    Bool	reverse,
-	    Bool	upsidedown,
-	    Pixel	bitplane,
-	    void	*closure);
-
-void
+extern _X_EXPORT void
 fbPushImage (DrawablePtr    pDrawable,
 	     GCPtr	    pGC,
 	     
@@ -1730,7 +1705,7 @@ fbPushImage (DrawablePtr    pDrawable,
 	     int	    width,
 	     int	    height);
 
-void
+extern _X_EXPORT void
 fbPushPixels (GCPtr	    pGC,
 	      PixmapPtr	    pBitmap,
 	      DrawablePtr   pDrawable,
@@ -1744,27 +1719,27 @@ fbPushPixels (GCPtr	    pGC,
  * fbscreen.c
  */
 
-Bool
+extern _X_EXPORT Bool
 fbCloseScreen (int indx, ScreenPtr pScreen);
 
-Bool
+extern _X_EXPORT Bool
 fbRealizeFont(ScreenPtr pScreen, FontPtr pFont);
 
-Bool
+extern _X_EXPORT Bool
 fbUnrealizeFont(ScreenPtr pScreen, FontPtr pFont);
 
-void
+extern _X_EXPORT void
 fbQueryBestSize (int class, 
 		 unsigned short *width, unsigned short *height,
 		 ScreenPtr pScreen);
 
-PixmapPtr
+extern _X_EXPORT PixmapPtr
 _fbGetWindowPixmap (WindowPtr pWindow);
 
-void
+extern _X_EXPORT void
 _fbSetWindowPixmap (WindowPtr pWindow, PixmapPtr pPixmap);
 
-Bool
+extern _X_EXPORT Bool
 fbSetupScreen(ScreenPtr	pScreen, 
 	      pointer	pbits,		/* pointer to screen bitmap */
 	      int	xsize, 		/* in pixels */
@@ -1774,7 +1749,7 @@ fbSetupScreen(ScreenPtr	pScreen,
 	      int	width,		/* pixel width of frame buffer */
 	      int	bpp);		/* bits per pixel of frame buffer */
 
-Bool
+extern _X_EXPORT Bool
 wfbFinishScreenInit(ScreenPtr	pScreen,
 		    pointer	pbits,
 		    int		xsize,
@@ -1786,7 +1761,7 @@ wfbFinishScreenInit(ScreenPtr	pScreen,
 		    SetupWrapProcPtr setupWrap,
 		    FinishWrapProcPtr finishWrap);
 
-Bool
+extern _X_EXPORT Bool
 wfbScreenInit(ScreenPtr	pScreen,
 	      pointer	pbits,
 	      int	xsize,
@@ -1798,7 +1773,7 @@ wfbScreenInit(ScreenPtr	pScreen,
 	      SetupWrapProcPtr setupWrap,
 	      FinishWrapProcPtr finishWrap);
 
-Bool
+extern _X_EXPORT Bool
 fbFinishScreenInit(ScreenPtr	pScreen,
 		   pointer	pbits,
 		   int		xsize,
@@ -1808,7 +1783,7 @@ fbFinishScreenInit(ScreenPtr	pScreen,
 		   int		width,
 		   int		bpp);
 
-Bool
+extern _X_EXPORT Bool
 fbScreenInit(ScreenPtr	pScreen,
 	     pointer	pbits,
 	     int	xsize,
@@ -1817,9 +1792,6 @@ fbScreenInit(ScreenPtr	pScreen,
 	     int	dpiy,
 	     int	width,
 	     int	bpp);
-
-void
-fbInitializeBackingStore (ScreenPtr pScreen);
     
 /*
  * fbseg.c
@@ -1837,12 +1809,12 @@ typedef void	FbBres (DrawablePtr	pDrawable,
 			int		e3,
 			int		len);
 
-FbBres fbBresSolid, fbBresDash, fbBresFill, fbBresFillDash;
+extern _X_EXPORT FbBres fbBresSolid, fbBresDash, fbBresFill, fbBresFillDash;
 /*
  * fbsetsp.c
  */
 
-void
+extern _X_EXPORT void
 fbSetSpans (DrawablePtr	    pDrawable,
 	    GCPtr	    pGC,
 	    char	    *src,
@@ -1851,11 +1823,11 @@ fbSetSpans (DrawablePtr	    pDrawable,
 	    int		    nspans,
 	    int		    fSorted);
 
-FbBres *
+extern _X_EXPORT FbBres *
 fbSelectBres (DrawablePtr   pDrawable,
 	      GCPtr	    pGC);
 
-void
+extern _X_EXPORT void
 fbBres (DrawablePtr	pDrawable,
 	GCPtr		pGC,
 	int		dashOffset,
@@ -1869,7 +1841,7 @@ fbBres (DrawablePtr	pDrawable,
 	int		e3,
 	int		len);
 
-void
+extern _X_EXPORT void
 fbSegment (DrawablePtr	pDrawable,
 	   GCPtr	pGC,
 	   int		xa,
@@ -1884,7 +1856,7 @@ fbSegment (DrawablePtr	pDrawable,
  * fbsolid.c
  */
 
-void
+extern _X_EXPORT void
 fbSolid (FbBits	    *dst,
 	 FbStride   dstStride,
 	 int	    dstX,
@@ -1897,7 +1869,7 @@ fbSolid (FbBits	    *dst,
 	 FbBits	    xor);
 
 #ifdef FB_24BIT
-void
+extern _X_EXPORT void
 fbSolid24 (FbBits   *dst,
 	   FbStride dstStride,
 	   int	    dstX,
@@ -1913,13 +1885,13 @@ fbSolid24 (FbBits   *dst,
  * fbstipple.c
  */
 
-void
+extern _X_EXPORT void
 fbTransparentSpan (FbBits   *dst,
 		   FbBits   stip,
 		   FbBits   fgxor,
 		   int	    n);
 
-void
+extern _X_EXPORT void
 fbEvenStipple (FbBits   *dst,
 	       FbStride dstStride,
 	       int	dstX,
@@ -1940,7 +1912,7 @@ fbEvenStipple (FbBits   *dst,
 	       int	xRot,
 	       int	yRot);
 
-void
+extern _X_EXPORT void
 fbOddStipple (FbBits	*dst,
 	      FbStride	dstStride,
 	      int	dstX,
@@ -1962,7 +1934,7 @@ fbOddStipple (FbBits	*dst,
 	      int	xRot,
 	      int	yRot);
 
-void
+extern _X_EXPORT void
 fbStipple (FbBits   *dst,
 	   FbStride dstStride,
 	   int	    dstX,
@@ -1989,7 +1961,7 @@ fbStipple (FbBits   *dst,
  * fbtile.c
  */
 
-void
+extern _X_EXPORT void
 fbEvenTile (FbBits	*dst,
 	    FbStride	dstStride,
 	    int		dstX,
@@ -2006,7 +1978,7 @@ fbEvenTile (FbBits	*dst,
 	    int		xRot,
 	    int		yRot);
 
-void
+extern _X_EXPORT void
 fbOddTile (FbBits	*dst,
 	   FbStride	dstStride,
 	   int		dstX,
@@ -2026,7 +1998,7 @@ fbOddTile (FbBits	*dst,
 	   int		xRot,
 	   int		yRot);
 
-void
+extern _X_EXPORT void
 fbTile (FbBits	    *dst,
 	FbStride    dstStride,
 	int	    dstX,
@@ -2049,37 +2021,37 @@ fbTile (FbBits	    *dst,
 /*
  * fbutil.c
  */
-FbBits
+extern _X_EXPORT FbBits
 fbReplicatePixel (Pixel p, int bpp);
 
-void
+extern _X_EXPORT void
 fbReduceRasterOp (int rop, FbBits fg, FbBits pm, FbBits *andp, FbBits *xorp);
 
 #ifdef FB_ACCESS_WRAPPER
-extern ReadMemoryProcPtr wfbReadMemory;
-extern WriteMemoryProcPtr wfbWriteMemory;
+extern _X_EXPORT ReadMemoryProcPtr wfbReadMemory;
+extern _X_EXPORT WriteMemoryProcPtr wfbWriteMemory;
 #endif
 
 /*
  * fbwindow.c
  */
 
-Bool
+extern _X_EXPORT Bool
 fbCreateWindow(WindowPtr pWin);
 
-Bool
+extern _X_EXPORT Bool
 fbDestroyWindow(WindowPtr pWin);
 
-Bool
+extern _X_EXPORT Bool
 fbMapWindow(WindowPtr pWindow);
 
-Bool
+extern _X_EXPORT Bool
 fbPositionWindow(WindowPtr pWin, int x, int y);
 
-Bool 
+extern _X_EXPORT Bool
 fbUnmapWindow(WindowPtr pWindow);
     
-void
+extern _X_EXPORT void
 fbCopyWindowProc (DrawablePtr	pSrcDrawable,
 		  DrawablePtr	pDstDrawable,
 		  GCPtr		pGC,
@@ -2092,32 +2064,24 @@ fbCopyWindowProc (DrawablePtr	pSrcDrawable,
 		  Pixel		bitplane,
 		  void		*closure);
 
-void 
+extern _X_EXPORT void
 fbCopyWindow(WindowPtr	    pWin, 
 	     DDXPointRec    ptOldOrg, 
 	     RegionPtr	    prgnSrc);
 
-Bool
+extern _X_EXPORT Bool
 fbChangeWindowAttributes(WindowPtr pWin, unsigned long mask);
 
-void
+extern _X_EXPORT void
 fbFillRegionSolid (DrawablePtr	pDrawable,
 		   RegionPtr	pRegion,
 		   FbBits	and,
 		   FbBits	xor);
 
-void
-fbFillRegionTiled (DrawablePtr	pDrawable,
-		   RegionPtr	pRegion,
-		   PixmapPtr	pTile);
-
-void
-fbPaintWindow(WindowPtr pWin, RegionPtr pRegion, int what);
-
-
-pixman_image_t *image_from_pict (PicturePtr pict,
-				 Bool       has_clip);
-void free_pixman_pict (PicturePtr, pixman_image_t *);
+extern _X_EXPORT pixman_image_t *
+image_from_pict (PicturePtr pict,
+		 Bool       has_clip);
+extern _X_EXPORT void free_pixman_pict (PicturePtr, pixman_image_t *);
 
 #endif /* _FB_H_ */
 
